@@ -5,7 +5,7 @@ if [ "$2" = "2" ]; then
     docker exec ndts /bin/bash -c 'export DISPLAY=":99.0"; python test'
 else
     echo "run python3-nxstools"
-    if [ "$1" = "debian10" ] || [ "$1" = "ubuntu22.04" ] || [ "$1" = "ubuntu20.04" ] || [ "$1" = "ubuntu20.10" ] || [ "$1" = "debian11" ] ; then
+    if [ "$1" = "debian10" ] || [ "$1" = "ubuntu22.04" ] || [ "$1" = "ubuntu24.04" ] || [ "$1" = "ubuntu24.10" ] || [ "$1" = "ubuntu23.10" ] || [ "$1" = "ubuntu20.04" ] || [ "$1" = "ubuntu20.10" ] || [ "$1" = "debian11" ]  || [ "$1" = "debian12" ]  || [ "$1" = "debian12tg10" ] ; then
 	docker exec ndts python3 -m pytest --cov=nxstools --cov-report=term-missing test
     else
 	if [ "$1" = "debian9" ]; then
