@@ -396,7 +396,7 @@ class SetUp(object):
 
     def waitServerNotRunning(self, server=None, device=None,
                              adminproxy=None,
-                             maxcnt=1000, verbose=True,
+                             maxcnt=10000, verbose=True,
                              waitforproc=True):
         """  wait until device is exported and server is running
 
@@ -428,7 +428,7 @@ class SetUp(object):
                     # running = adminproxy.RunningServers
                     running = adminproxy.DevGetRunningServers(True)
                     if server in running:
-                        time.sleep(0.01)
+                        time.sleep(0.02)
                         cnt += 1
                         continue
                     else:
@@ -439,7 +439,7 @@ class SetUp(object):
                         sys.stdout.flush()
                     exl = self.db.get_device_exported(device)
                     if device in exl.value_string:
-                        time.sleep(0.01)
+                        time.sleep(0.02)
                         cnt += 1
                         continue
                 found = False
@@ -448,7 +448,7 @@ class SetUp(object):
                         print(" %s is not working" % (device or server))
             except Exception as e:
                 print(str(e))
-                time.sleep(0.01)
+                time.sleep(0.02)
                 found = True
             cnt += 1
         if waitforproc:
@@ -457,7 +457,7 @@ class SetUp(object):
 
     def waitServerRunning(self, server=None, device=None,
                           adminproxy=None,
-                          maxcnt=1000, verbose=True,
+                          maxcnt=10000, verbose=True,
                           waitforproc=True):
         """  wait until device is exported and server is running
 
@@ -487,7 +487,7 @@ class SetUp(object):
                         sys.stdout.flush()
                     exl = self.db.get_device_exported(device)
                     if device not in exl.value_string:
-                        time.sleep(0.01)
+                        time.sleep(0.02)
                         cnt += 1
                         continue
                     else:
@@ -500,7 +500,7 @@ class SetUp(object):
                     # running = adminproxy.RunningServers
                     running = adminproxy.DevGetRunningServers(True)
                     if server not in running:
-                        time.sleep(0.01)
+                        time.sleep(0.02)
                         cnt += 1
                         continue
                 found = True
@@ -509,7 +509,7 @@ class SetUp(object):
                         print(" %s is working" % (device or server))
             except Exception as e:
                 print(str(e))
-                time.sleep(0.01)
+                time.sleep(0.02)
                 found = False
             cnt += 1
         if waitforproc:
@@ -551,7 +551,9 @@ class SetUp(object):
                         shell=True)
             pipe.close()
 
-    def restartServer(self, name, host=None, level=None, restart=True):
+    def restartServer(self, name, host=None, level=None,
+                      restart=True, stopstart=True, wait=True,
+                      timeout=None):
         """ restarts server
 
         :param name: server name
@@ -562,6 +564,12 @@ class SetUp(object):
         :type level: :obj:`int`
         :param restart:  if server should be restarted
         :type restart: :obj:`bool`
+        :param stopstart:  if server should be stopped and started
+        :type stopstart: :obj:`bool`
+        :param wait:  script should wait for the server
+        :type wait: :obj:`bool`
+        :param timeout: timeout for  start
+        :type timeout: :obj:`float`
         """
         if name:
             admin = self.getStarterName(host)
@@ -592,40 +600,52 @@ class SetUp(object):
                                 if level is not None:
                                     self._changeLevel(
                                         svl, level, tohigher=False)
-                                if started and svl in started:
+                                problems = False
+                                if stopstart:
+                                    if started and svl in started:
+                                        try:
+                                            # adminproxy.HardKillServer(svl)
+                                            adminproxy.DevStop(svl)
+                                            # self.killServer(svl)
+                                        except Exception:
+                                            adminproxy.HardKillServer(svl)
+                                        problems = self.waitServerNotRunning(
+                                            svl, None, adminproxy,
+                                            verbose=None)
+                                        if problems:
+                                            self.killServer(svl)
+                                            print("Server Running")
+                                        sys.stdout.write(
+                                            "Restarting: %s" % svl)
+                                    else:
+                                        sys.stdout.write("Starting: %s" % svl)
+                                    sys.stdout.flush()
+                                    problems = True
+                                    counter = 0
+                                    while problems and counter < 1000:
+                                        try:
+                                            sys.stdout.write('.')
+                                            sys.stdout.flush()
+                                            adminproxy.DevStart(svl)
+                                            problems = False
+                                        except Exception:
+                                            counter += 1
+                                            time.sleep(0.2)
+                                if wait:
                                     try:
-                                        # adminproxy.HardKillServer(svl)
-                                        adminproxy.DevStop(svl)
-                                        # self.killServer(svl)
+                                        maxcnt = int(float(timeout)/0.02)
                                     except Exception:
-                                        adminproxy.HardKillServer(svl)
-                                    problems = self.waitServerNotRunning(
-                                        svl, None, adminproxy, verbose=None)
+                                        maxcnt = 10000
+                                    problems = not self.waitServerRunning(
+                                        svl, None, adminproxy, maxcnt) \
+                                        or problems
                                     if problems:
-                                        self.killServer(svl)
-                                        print("Server Running")
-                                    sys.stdout.write("Restarting: %s" % svl)
+                                        print("%s was not restarted" % svl)
+                                        print("Warning: "
+                                              "Process with the server"
+                                              "instance could be suspended")
                                 else:
-                                    sys.stdout.write("Starting: %s" % svl)
-                                sys.stdout.flush()
-                                problems = True
-                                counter = 0
-                                while problems and counter < 100:
-                                    try:
-                                        sys.stdout.write('.')
-                                        sys.stdout.flush()
-                                        adminproxy.DevStart(svl)
-                                        problems = False
-                                    except Exception:
-                                        counter += 1
-                                        time.sleep(0.2)
-                                problems = not self.waitServerRunning(
-                                    svl, None, adminproxy) or problems
-                                print(" ")
-                                if problems:
-                                    print("%s was not restarted" % svl)
-                                    print("Warning: Process with the server"
-                                          "instance could be suspended")
+                                    print("")
 
     def __exported_servers(self):
         """ returns Servers for exported devices
@@ -713,7 +733,7 @@ class SetUp(object):
         self.db.put_server_info(sinfo)
         return True
 
-    def _startupServer(self, new, level, host, ctrl, device):
+    def _startupServer(self, new, level, host, ctrl, device, postpone=False):
         """ starts the server up
 
         :param new: new server name
@@ -726,6 +746,8 @@ class SetUp(object):
         :type ctrl: :obj:`str`
         :param device: device name
         :type device: :obj:`str`
+        :param postpone: postpone start server
+        :type postpone: :obj:`bool`
         :returns: True if server was started up
         :rtype: :obj:`bool`
         """
@@ -761,24 +783,27 @@ class SetUp(object):
         adminproxy.UpdateServersInfo()
         running = adminproxy.DevGetRunningServers(True)
         # running = adminproxy.RunningServers
-        if new not in running:
+        if new not in running and not postpone:
             adminproxy.DevStart(new)
         else:
             print("%s NOT STARTED" % new)
 
         adminproxy.UpdateServersInfo()
 
-        sys.stdout.write("waiting for server ")
-        sys.stdout.flush()
-        return self.waitServerRunning(new, device, adminproxy)
+        if not postpone:
+            sys.stdout.write("waiting for server ")
+            sys.stdout.flush()
+            return self.waitServerRunning(new, device, adminproxy)
 
-    def createDataWriter(self, beamline, masterhost):
+    def createDataWriter(self, beamline, masterhost, postpone=False):
         """ creates data writer
 
         :param beamline: beamline name
         :type beamline: :obj:`str`
         :param masterhost: master host of data writer
         :type masterhost: :obj:`str`
+        :param postpone: postpone starting flag
+        :type postpone: :obj:`bool`
         :returns: True if server was created
         :rtype: :obj:`bool`
         """
@@ -818,11 +843,89 @@ class SetUp(object):
 
         hostname = socket.gethostname()
 
-        self._startupServer(full_class_name, 1, hostname, 1, self.writer_name)
+        self._startupServer(full_class_name, 1, hostname, 1, self.writer_name,
+                            postpone)
 
         return True
 
-    def createConfigServer(self, beamline, masterhost, jsonsettings=None):
+    def createServer(self, server_name, beamline, masterhost, hostname=None,
+                     class_name=None, jsondeviceproperties=None,
+                     postpone=False):
+        """ creates tango server in DB
+
+        :param server_name: server name
+        :type server_name: :obj:`str`
+        :param beamline: beamline name
+        :type beamline: :obj:`str`
+        :param masterhost: master host of data writer
+        :type masterhost: :obj:`str`
+        :param hostname: host to run the config server
+        :type hostname: :obj:`str`
+        :param class_name: class name
+        :type clas_name: :obj:`str`
+        :param jsondeviceproperties: json device properties
+        :type jsondeviceproperties: :obj:`str`
+        :param postpone: postpone starting flag
+        :type postpone: :obj:`bool`
+        :returns: True if server was created
+        :rtype: :obj:`bool`
+        """
+        if not beamline:
+            print("createServer: no beamline given ")
+            return False
+        if not masterhost:
+            print("createServer: no masterhost given ")
+            return False
+
+        if "/" not in server_name:
+            server_name = server_name + '/' + masterhost
+        server = server_name.split("/")[0]
+        if not class_name:
+            class_name = server
+        self.device_name = "%s/%s/%s" % (beamline, server.lower(), masterhost)
+        if server_name not in self.db.get_server_list(server_name):
+            print("creating: %s" % server_name)
+
+            if server_name in self.db.get_server_list(server_name):
+                print("createDataWriter: DB contains already %s" % server_name)
+                return False
+
+            try:
+                properties = dict(json.loads(jsondeviceproperties))
+            except Exception:
+                print("\ncreateServer: properties cannot be load")
+                properties = {}
+            if not properties:
+                properties = {self.device_name: {}}
+
+            print("PROP", properties)
+            for dv, props in properties.items():
+                di = tango.DbDevInfo()
+                di.name = dv
+                di._class = class_name
+                di.server = server_name
+                self.db.add_device(di)
+                try:
+                    self.db.put_device_property(dv, props)
+                except Exception:
+                    print("\ncreateServer: properties cannot be added")
+
+        elif (self.device_name not in
+              self.db.get_device_class_list(server_name).value_string):
+            print("\ncreateServer: %s already exists. "
+                  "To change its device name please remove it." % server_name)
+            return False
+
+        if not hostname:
+            hostname = socket.gethostname()
+
+        self._startupServer(server_name, 1, hostname, 1, self.device_name,
+                            postpone)
+
+        return True
+
+    def createConfigServer(self, beamline, masterhost, jsonsettings=None,
+                           hostname=None, postpone=False):
         """ creates configuration server
 
         :param beamline: beamline name
@@ -831,6 +934,10 @@ class SetUp(object):
         :type masterhost: :obj:`str`
         :param jsonsettings: connection settings to DB in json
         :type jsonsettings: :obj:`str`
+        :param hostname: host to run the config server
+        :type hostname: :obj:`str`
+        :param postpone: postpone starting flag
+        :type postpone: :obj:`bool`
         :returns: True if server was created
         :rtype: :obj:`bool`
         """
@@ -868,42 +975,46 @@ class SetUp(object):
                   "To change its device name please remove it." % server_name)
             return False
 
-        hostname = self.db.get_db_host().split(".")[0]
+        if not hostname:
+            hostname = self.db.get_db_host().split(".")[0]
 
-        self._startupServer(server_name, 1, hostname, 1, self.cserver_name)
+        self._startupServer(server_name, 1, hostname, 1, self.cserver_name,
+                            postpone)
 
-        dp = tango.DeviceProxy(self.cserver_name)
-        if dp.state() != tango.DevState.ON:
-            dp.Close()
-        if jsonsettings:
+        if not postpone:
             dp = tango.DeviceProxy(self.cserver_name)
-            dp.JSONSettings = jsonsettings
-        try:
-            dp.Open()
-        except Exception:
+            if dp.state() != tango.DevState.ON:
+                dp.Close()
+            if jsonsettings:
+                dp = tango.DeviceProxy(self.cserver_name)
+                dp.JSONSettings = jsonsettings
             try:
-                jsettings = json.loads(jsonsettings)
-                jsettings['read_default_file'] = \
-                    '/var/lib/nxsconfigserver/.my.cnf'
-                dp.JSONSettings = str(json.dumps(jsettings))
                 dp.Open()
             except Exception:
                 try:
+                    jsettings = json.loads(jsonsettings)
                     jsettings['read_default_file'] = \
                         '/var/lib/nxsconfigserver/.my.cnf'
                     dp.JSONSettings = str(json.dumps(jsettings))
                     dp.Open()
                 except Exception:
-                    print("createConfigServer: "
-                          "%s cannot connect the"
-                          " database with JSONSettings: \n%s " % (
-                              self.cserver_name, jsonsettings))
-                    print("try to change the settings")
-                    return False
+                    try:
+                        jsettings['read_default_file'] = \
+                            '/var/lib/nxsconfigserver/.my.cnf'
+                        dp.JSONSettings = str(json.dumps(jsettings))
+                        dp.Open()
+                    except Exception:
+                        print("createConfigServer: "
+                              "%s cannot connect the"
+                              " database with JSONSettings: \n%s " % (
+                                  self.cserver_name, jsonsettings))
+                        print("try to change the settings")
+                        return False
 
         return True
 
-    def createSelector(self, beamline, masterhost, writer=None, cserver=None):
+    def createSelector(self, beamline, masterhost, writer=None, cserver=None,
+                       postpone=False):
         """ creates selector server
 
         :param beamline: beamline name
@@ -914,6 +1025,8 @@ class SetUp(object):
         :type writer: :obj:`str`
         :param cserver: configuration server device name
         :type cserver: :obj:`str`
+        :param postpone: postpone starting flag
+        :type postpone: :obj:`bool`
         :returns: True if server was created
         :rtype: :obj:`bool`
         """
@@ -954,16 +1067,18 @@ class SetUp(object):
 
         hostname = socket.gethostname()
 
-        self._startupServer(full_class_name, 4, hostname, 1, device_name)
+        self._startupServer(full_class_name, 4, hostname, 1,
+                            device_name, postpone)
 
-        if self.writer_name or self.cserver_name:
-            dp = tango.DeviceProxy(device_name)
-            dp.ping()
-            self.waitServerRunning(None, device_name)
-            if self.cserver_name:
-                dp.configDevice = self.cserver_name
-            if self.writer_name:
-                dp.writerDevice = self.writer_name
+        if not postpone:
+            if self.writer_name or self.cserver_name:
+                dp = tango.DeviceProxy(device_name)
+                dp.ping()
+                self.waitServerRunning(None, device_name)
+                if self.cserver_name:
+                    dp.configDevice = self.cserver_name
+                if self.writer_name:
+                    dp.writerDevice = self.writer_name
         return True
 
 
@@ -980,6 +1095,10 @@ class Set(Runner):
         + "       nxsetup set\n" \
         + "       nxsetup set -b p09 -m haso228 -u p09user " \
         + "-d nxsconfig NXSConfigServer\n" \
+        + "       nxsetup set NexusWriter/haso228  -k NexusWriter  " \
+        + "-y '{\"p00/bliss_nexuswriter/test_session\":" \
+        + "{\"session\":\"test_session\"," \
+        + "\"beacon_host\":\"haso228:25000\"}}'  -t\n" \
         + "\n"
 
     def create(self):
@@ -994,6 +1113,14 @@ class Set(Runner):
             "-m", "--masterhost", action="store",
             dest="masterhost", help="the host that stores the Mg"
             " ( default: <localhost> )")
+        parser.add_argument(
+            "-c", "--confighost", action="store",
+            dest="confighost", help="the host run the config server"
+            " ( default: <mysqlhost> )")
+        parser.add_argument(
+            "-r", "--runhost", action="store",
+            dest="runhost", help="the host to run the server"
+            " ( default: localhost )")
         parser.add_argument(
             "-u", "--user", action="store",
             dest="user", help="the local user"
@@ -1012,6 +1139,18 @@ class Set(Runner):
             "  where <MY_CNF_FILE> stays for"
             " \"/home/<USER>/.my.cnf\""
             " or \"/var/lib/nxsconfigserver/.my.cnf\" )")
+        parser.add_argument(
+            "-k", "--class-name", action="store",
+            dest="classname", help="tango server class name")
+        parser.add_argument(
+            "-y", "--json-device-properties", action="store",
+            dest="propjson",
+            help="JSON tango device properties "
+            "( default: '{}' )")
+        parser.add_argument(
+            "-t", "--postpone", action="store_true",
+            default=False, dest="postpone",
+            help="do not start the server")
         parser.add_argument(
             'args', metavar='server_name',
             type=str, nargs='*',
@@ -1078,20 +1217,41 @@ class Set(Runner):
                 + ' "read_default_file":"/home/%s/.my.cnf",' % options.user \
                 + ' "use_unicode":true}'
 
+        if options.propjson:
+            jsondeviceproperties = options.propjson
+        else:
+            jsondeviceproperties = '{}'
+
         if not args or "NXSConfigServer" in args:
             if not setUp.createConfigServer(
                     beamline=options.beamline,
                     masterhost=options.masterhost,
-                    jsonsettings=jsonsettings):
+                    jsonsettings=jsonsettings,
+                    hostname=options.confighost,
+                    postpone=options.postpone):
                 print("startup failed to create the nexus config server")
                 sys.exit(255)
 
         if not args or "NXSRecSelector" in args:
             if not setUp.createSelector(
                     beamline=options.beamline,
-                    masterhost=options.masterhost):
+                    masterhost=options.masterhost,
+                    postpone=options.postpone):
                 print("startup failed to create the nexus selector server")
                 sys.exit(255)
+        for arg in args:
+            if arg not in ["NXSDataWriter", "NXSRecSelector",
+                           "NXSConfigServer"]:
+                if not setUp.createServer(
+                        arg,
+                        beamline=options.beamline,
+                        masterhost=options.masterhost,
+                        hostname=options.runhost,
+                        class_name=options.classname,
+                        jsondeviceproperties=jsondeviceproperties,
+                        postpone=options.postpone):
+                    print("startup failed to create the %s server" % arg)
+                    sys.exit(255)
 
 
 class Start(Runner):
@@ -1113,6 +1273,13 @@ class Start(Runner):
         parser.add_argument(
             "-l", "--level", action="store", type=int, default=-1,
             dest="level", help="startup level")
+        parser.add_argument(
+            "-z", "--timeout", action="store", type=float, default=None,
+            dest="timeout", help="timeout in seconds")
+        parser.add_argument(
+            "-e", "--no-wait", action="store_true",
+            default=False, dest="nowait",
+            help="do not wait")
         parser.add_argument(
             'args', metavar='server_name',
             type=str, nargs='*',
@@ -1132,7 +1299,48 @@ class Start(Runner):
             setUp.restartServer(
                 server,
                 level=(options.level if options.level > -1 else None),
-                restart=False)
+                restart=False, timeout=options.timeout,
+                wait=(not options.nowait))
+
+
+class Wait(Runner):
+
+    """ start runner"""
+
+    #: (:obj:`str`) command description
+    description = "wait for tango server"
+    #: (:obj:`str`) command epilog
+    epilog = "" \
+        + " examples:\n" \
+        + "       nxsetup wait Pool/haso228 -z 30\n" \
+        + "\n"
+
+    def create(self):
+        """ creates parser
+        """
+        parser = self._parser
+        parser.add_argument(
+            "-z", "--timeout", action="store", type=float, default=None,
+            dest="timeout", help="timeout in seconds")
+        parser.add_argument(
+            'args', metavar='server_name',
+            type=str, nargs='*',
+            help='server names, e.g.: NXSRecSelector NXSDataWriter/TDW1')
+
+    def run(self, options):
+        """ the main program function
+
+        :param options: parser options
+        :type options: :class:`argparse.Namespace`
+        """
+        args = options.args or []
+        setUp = SetUp()
+        servers = args if args else [
+            "NXSConfigServer", "NXSRecSelector", "NXSDataWriter"]
+        for server in servers:
+            setUp.restartServer(
+                server,
+                restart=False, stopstart=False, timeout=options.timeout)
 
 
 class MoveProp(Runner):
@@ -1164,6 +1372,13 @@ class MoveProp(Runner):
             "-t", "--postpone", action="store_true",
             default=False, dest="postpone",
             help="do not restart the server")
+        parser.add_argument(
+            "-z", "--timeout", action="store", type=float, default=None,
+            dest="timeout", help="timeout in seconds")
+        parser.add_argument(
+            "-e", "--no-wait", action="store_true",
+            default=False, dest="nowait",
+            help="do not wait")
         parser.add_argument(
             'args', metavar='server_name',
             type=str, nargs='*',
@@ -1186,7 +1401,9 @@ class MoveProp(Runner):
             if setUp.changePropertyName(
                     server, options.oldname, options.newname):
                 if not options.postpone:
-                    setUp.restartServer(server)
+                    setUp.restartServer(
+                        server, timeout=options.timeout,
+                        wait=(not options.nowait))
 
 
 class ChangeProp(Runner):
@@ -1221,6 +1438,13 @@ class ChangeProp(Runner):
             "-t", "--postpone", action="store_true",
             default=False, dest="postpone",
             help="do not restart the server")
+        parser.add_argument(
+            "-z", "--timeout", action="store", type=float, default=None,
+            dest="timeout", help="timeout in seconds")
+        parser.add_argument(
+            "-e", "--no-wait", action="store_true",
+            default=False, dest="nowait",
+            help="do not wait")
         parser.add_argument(
             'args', metavar='server_name',
             type=str, nargs='*',
@@ -1243,7 +1467,9 @@ class ChangeProp(Runner):
             if setUp.changePropertyValue(
                     server, options.newname, options.propvalue):
                 if not options.postpone:
-                    setUp.restartServer(server)
+                    setUp.restartServer(
+                        server,
+                        timeout=options.timeout, wait=(not options.nowait))
 
 
 class AddRecorderPath(Runner):
@@ -1271,11 +1497,18 @@ class AddRecorderPath(Runner):
             "-t", "--postpone", action="store_true",
             default=False, dest="postpone",
             help="do not restart the server")
+        parser.add_argument(
+            "-z", "--timeout", action="store", type=float, default=None,
+            dest="timeout", help="timeout in seconds")
+        parser.add_argument(
+            "-e", "--no-wait", action="store_true",
+            default=False, dest="nowait",
+            help="do not wait")
         parser.add_argument(
             "-i", "--instance", action="store",
             dest="instance",
             help="macroserver instance name, i.e. haso"
-            " ( default: '*'")
+            " ( default: '*')")
 
     def postauto(self):
         """ creates parser
@@ -1296,16 +1529,15 @@ class AddRecorderPath(Runner):
             options.instance = "*"
 
         setUp = SetUp()
-        print("RUN")
         if setUp.changeRecorderPath(
                 options.recpath[0], options.instance):
-            print("RUN2")
             if not options.postpone:
                 if options.instance != "*":
                     ms = "MacroServer/%s" % options.instance
                 else:
                     ms = "MacroServer"
-                setUp.restartServer(ms)
+                setUp.restartServer(ms, timeout=options.timeout,
+                                    wait=(not options.nowait))
 
 
 class Restart(Runner):
@@ -1327,6 +1559,13 @@ class Restart(Runner):
         parser.add_argument(
             "-l", "--level", action="store", type=int, default=-1,
             dest="level", help="startup level")
+        parser.add_argument(
+            "-z", "--timeout", action="store", type=float, default=None,
+            dest="timeout", help="timeout in seconds")
+        parser.add_argument(
+            "-e", "--no-wait", action="store_true",
+            default=False, dest="nowait",
+            help="do not wait")
         parser.add_argument(
             'args', metavar='server_name',
             type=str, nargs='*',
@@ -1344,7 +1583,8 @@ class Restart(Runner):
             "NXSConfigServer", "NXSRecSelector", "NXSDataWriter"]
         for server in servers:
             setUp.restartServer(
-                server, level=(options.level if options.level > -1 else None))
+                server, level=(options.level if options.level > -1 else None),
+                timeout=options.timeout, wait=(not options.nowait))
 
 
 class Stop(Runner):
@@ -1437,6 +1677,7 @@ def main():
         ('restart', Restart),
         ('start', Start),
         ('stop', Stop),
+        ('wait', Wait),
         ('move-prop', MoveProp),
         ('change-prop', ChangeProp),
         ('add-recorder-path', AddRecorderPath),
