@@ -24,9 +24,10 @@ import json
 import sys
 import xml.etree.ElementTree as et
 import numpy as np
-import pytz
+import math
 import time
 import dateutil.parser
+import re
 from lxml.etree import XMLParser
 
 from nxstools.nxsparser import ParserTools
@@ -35,6 +36,7 @@ from nxstools.nxsparser import ParserTools
 class numpyEncoder(json.JSONEncoder):
     """ numpy json encoder with list
     """
+
     def default(self, obj):
         """ default encoder
 
@@ -57,6 +59,75 @@ class numpyEncoder(json.JSONEncoder):
         return json.JSONEncoder.default(self, obj)
 
 
+class numpyEncoderNull(numpyEncoder):
+    """ numpy json encoder with list with nan/inf to null
+    """
+    def encode(self, obj, *args, **kwargs):
+        return numpyEncoder.encode(self, infNaN2None(obj), *args, **kwargs)
+
+
+def infNaN2None(obj):
+    """ replace inf and NaN to None
+    """
+    if isinstance(obj, dict):
+        return {ky: infNaN2None(vl) for ky, vl in obj.items()}
+    elif isinstance(obj, list):
+        return [infNaN2None(it) for it in obj]
+    elif isinstance(obj, float) and math.isinf(obj):
+        return None
+    elif isinstance(obj, float) and math.isnan(obj):
+        return None
+    return obj
+
+
+_regex = r'^(-?(?:[1-9][0-9]*)?[0-9]{4})-(1[0-2]|0[1-9])-' \
+    r'(3[01]|0[1-9]|[12][0-9])T(2[0-3]|[01][0-9]):([0-5][0-9]):([0-5][0-9])' \
+    r'(\.[0-9]+)?(Z|[+-](?:2[0-3]|[01][0-9]):[0-5][0-9])?$'
+
+_match_iso8601 = re.compile(_regex).match
+
+
+def isoDate(text):
+    """ convert date to iso format
+
+    :param text: date text to convert
+    :type text: :obj:`str`
+    :returns: date in iso format
+    :rtype: :obj:`str`
+    """
+    result = ""
+    try:
+        try:
+            if _match_iso8601(text) is not None:
+                result = text
+        except Exception:
+            pass
+        if not result:
+            date = dateutil.parser.parse(text)
+            fmt = '%Y-%m-%dT%H:%M:%S.%f%z'
+            if date.tzinfo is None:
+                tzone = time.tzname[0]
+                if tzone in ['CET', 'CEST']:
+                    tzone = 'Europe/Berlin'
+                try:
+                    if sys.version_info >= (3, 9):
+                        import zoneinfo
+                        tz = zoneinfo.ZoneInfo(tzone)
+                        date = date.replace(tzinfo=tz)
+                    else:
+                        import pytz
+                        tz = pytz.timezone(tzone)
+                        date = tz.localize(date)
+                except Exception:
+                    import tzlocal
+                    tz = tzlocal.get_localzone()
+                    date = date.replace(tzinfo=tz)
+            result = str(date.strftime(fmt))
+    except Exception:
+        result = text
+    return result
+
+
 def getdsname(xmlstring):
     """ provides datasource name from datasource xml string
 
@@ -485,7 +556,10 @@ class NXSFileParser(object):
                         if self.maxonedsize >= 0 and len(desc["shape"]) == 1 \
                            and hasattr(vl, "__len__") and \
                            len(vl) > self.maxonedsize:
-                            nd["value"] = [vl[0], vl[-1]]
+                            try:
+                                nd["value"] = [min(vl), max(vl)]
+                            except Exception:
+                                nd["value"] = [vl[0], vl[-1]]
                         else:
                             nd["value"] = vl
                         if self.emptyunits and "unit" not in nd.keys():
@@ -532,10 +606,12 @@ class NXSFileParser(object):
         """
         for entry in self.__root:
             nm = entry.name
+            at = None
             try:
-                at = entry.attributes["NX_class"]
+                if "NX_class" in entry.attributes.names():
+                    at = entry.attributes["NX_class"]
             except Exception:
-                at = None
+                pass
             if len(self.entryclasses) == 0 or \
                at and (filewriter.first(at.read()) in self.entryclasses):
                 if len(self.entrynames) == 0 or \
@@ -592,44 +668,19 @@ class FIOFileParser(object):
                 sline = line.split("Acquisition started at ")
                 if sline and sline[-1].strip():
                     meta["start_time"] = {
-                        "value": self._isoDate(sline[-1].strip()),
+                        "value": isoDate(sline[-1].strip()),
                         "unit": ""
                     }
             elif "Acquisition ended at " in line:
                 sline = line.split("Acquisition ended at ")
                 if sline and sline[-1].strip():
                     meta["end_time"] = {
-                        "value": self._isoDate(sline[-1].strip()),
+                        "value": isoDate(sline[-1].strip()),
                         "unit": ""
                     }
         if comments:
             meta["comments"] = comments
 
-    @classmethod
-    def _isoDate(cls, text):
-        """ convert date to iso format
-
-        :param text: date text to convert
-        :type text: :obj:`str`
-        :returns: date in iso format
-        :rtype: :obj:`str`
-        """
-        try:
-            date = dateutil.parser.parse(text)
-            if date.tzinfo is None:
-                tzone = time.tzname[0]
-                try:
-                    tz = pytz.timezone(tzone)
-                except Exception:
-                    import tzlocal
-                    tz = tzlocal.get_localzone()
-                date = tz.localize(date)
-            fmt = '%Y-%m-%dT%H:%M:%S.%f%z'
-            result = str(date.strftime(fmt))
-        except Exception:
-            result = text
-        return result
-
     def _appendParameters(self, lines, meta):
         """append comments
 
@@ -644,12 +695,33 @@ class FIOFileParser(object):
                 sline = line.split("=")
                 if len(sline) > 1 and sline[0].strip() and \
                    sline[1].strip():
-                    try:
-                        params[sline[0].strip().replace(" ", "_")] = \
-                            eval(sline[1].strip())
-                    except Exception:
-                        params[sline[0].strip().replace(" ", "_")] = \
-                            str(sline[1].strip())
+                    if '@' not in sline[0]:
+                        try:
+                            params[sline[0].strip().replace(" ", "_")] = \
+                                eval(sline[1].strip())
+                        except Exception:
+                            params[sline[0].strip().replace(" ", "_")] = \
+                                str(sline[1].strip())
+        for line in lines:
+            if not line.startswith("!") and "=" in line:
+                sline = line.split("=")
+                if len(sline) > 1 and sline[0].strip() and \
+                   sline[1].strip():
+                    if '@' in sline[0]:
+                        field, attr = sline[0].strip().replace(" ", "_"). \
+                            split("@")[:2]
+                        try:
+                            avl = eval(sline[1].strip())
+                        except Exception:
+                            avl = str(sline[1].strip())
+
+                        if field in params:
+                            if not isinstance(params[field], dict):
+                                params[field] = {"value": params[field]}
+                            if attr in ["unit", "units"]:
+                                params[field]["unit"] = avl
+                            else:
+                                params[field][attr] = avl
 
         if params:
             meta["parameters"] = params
