@@ -25,10 +25,10 @@ import sys
 import random
 import struct
 import binascii
-# import time
-# import threading
-import PyTango
-# import json
+try:
+    import tango
+except Exception:
+    import PyTango as tango
 from nxstools import nxscreate
 
 try:
@@ -72,7 +72,7 @@ class NXSCreateTangoDSFSTest(unittest.TestCase):
                       '"read_default_file":"/etc/my.cnf", "use_unicode":true}'
 
         # home = expanduser("~")
-        db = PyTango.Database()
+        db = tango.Database()
         self.host = db.get_db_host().split(".")[0]
         self.port = db.get_db_port()
         self.directory = "."
